var expect = require('chai').expect;
var sinon = require('sinon');
var language = require('../../../models/language');
var express = require('express');

describe('languages routes tests', function() {
  var sandbox;
  var router;
  
  beforeEach(function() {
    sandbox = sinon.sandbox.create();
    
    sandbox.stub(express, 'Router').returns({
      get: sandbox.spy(),
      post: sandbox.spy(),
      delete: sandbox.spy()
    });

    router = require('../../../routes/languages');
  });
  
  afterEach(function() {
    sandbox.restore();
  });

  it('should register URI / for get', function() {
    expect(router.get.calledWith('/', sandbox.match.any)).to.be.true;
  });

  var stubResSend = function(expected, done) {
    return { send: function(data) {
      expect(data).to.be.eql(expected);
      done();
    }};
  }
  
  it("get / handler should call model's all & returns result", function(done) {
    var sampleLanguages = [{name: 'l1', author: 'a1'}];
    
    sandbox.stub(language, 'all', function(callback) {
      callback(null, sampleLanguages);
    });
  
    var req = {};
    var res = stubResSend(sampleLanguages, done);

    var registeredCallback = router.get.firstCall.args[1];
    registeredCallback(req, res);
  });


  it('should register URI / for post', function() {
    expect(router.post.calledWith('/', sandbox.match.any)).to.be.true;
  });

  it("post / handler should call model's add & returns sucess message", function(done) {
    var sampleLanguage = {name: 'l1', author: 'a1'};
    
    sandbox.stub(language, 'add', function(newLanguage, callback) {
      expect(newLanguage).to.be.eql(sampleLanguage);
      callback(null);
    });
    
    var req = { body: sampleLanguage };
    var res = stubResSend('language added', done);

    var registeredCallback = router.post.firstCall.args[1];
    registeredCallback(req, res);
  });

  it('should register URI /:id for delete', function() {
    expect(router.delete.calledWith('/:id', sandbox.match.any)).to.be.true;
  });

  it("delete /:validid handler should call model's delete, returns success", function(done) {
    sandbox.stub(language, 'delete', function(id, callback) {
      expect(id).to.be.eql(req.params.id);
      callback(null);
    });
    
    var req = {params: {id: 1}};    
    var res = stubResSend('language deleted', done);

    var registeredCallback = router.delete.firstCall.args[1];
    registeredCallback(req, res);
  });

  it("delete /:invalidid handler should return error message", function(done) {
    sandbox.stub(language, 'delete', function(id, callback) {
      expect(id).to.be.eql(req.params.id);
      callback(new Error('unable to delete language with id: 2319'));
    });
    
    var req = {params: {id: 2319}};    
    var res = stubResSend('unable to delete language with id: 2319', done);

    var registeredCallback = router.delete.firstCall.args[1];
    registeredCallback(req, res);
  });
});